@@ -123,6 +123,10 @@ import Data.Foldable (Foldable, foldr, foldl)
 import Data.Maybe (fromMaybe)
 import Data.PQueue.Prio.Max.Internals
 
+#if MIN_VERSION_base(4,9,0)
+import Data.Semigroup (Semigroup((<>)))
+#endif
+
 import Prelude hiding (map, filter, break, span, takeWhile, dropWhile, splitAt, take, drop, (!!), null, foldr, foldl)
 
 import qualified Data.PQueue.Prio.Min as Q
@@ -138,6 +142,11 @@ build f = f (:) []
 first' :: (a -> b) -> (a, c) -> (b, c)
 first' f (a, c) = (f a, c)
 
+#if MIN_VERSION_base(4,9,0)
+instance Ord k => Semigroup (MaxPQueue k a) where
+  (<>) = union
+#endif
+
 instance Ord k => Monoid (MaxPQueue k a) where
   mempty = empty
   mappend = union
@@ -185,8 +194,10 @@ singleton k a = MaxPQ (Q.singleton (Down k) a)
 insert :: Ord k => k -> a -> MaxPQueue k a -> MaxPQueue k a
 insert k a (MaxPQ q) = MaxPQ (Q.insert (Down k) a q)
 
--- | Amortized /O(1)/, worst-case /O(log n)/.  Insert an element into the priority queue,
---   putting it behind elements that compare equal to the inserted one.
+-- | /O(n)/ (an earlier implementation had /O(1)/ but was buggy).
+--   Insert an element with the specified key into the priority queue,
+--   putting it behind elements whos key compares equal to the
+--   inserted one.
 insertBehind :: Ord k => k -> a -> MaxPQueue k a -> MaxPQueue k a
 insertBehind k a (MaxPQ q) = MaxPQ (Q.insertBehind (Down k) a q)
 
